@@ -141,7 +141,7 @@ static char *make_url(t_session *session, char *request_uri) {
 	}
 
 	memcpy(url, *(session->host->hostname.item), len);
-	strncpy(url + len, request_uri, len_ru + 1);
+	memcpy(url + len, request_uri, len_ru + 1);
 
 	return url;
 }
@@ -159,7 +159,7 @@ static void secure_header(char *buffer, int size) {
 	/* Remove cookies
 	 */
 	while ((pos = strncasestr(buffer, "Set-Cookie:", header_length)) != NULL) {
-		strcpy(pos, "X-Empty: ");
+		memcpy(pos, "X-Empty: ", 9);
 		pos += 9;
 		do {
 			*(pos++) = 'x';
