@@ -22,7 +22,7 @@
 #include "session.h"
 #include "libstr.h"
 #include "challenge.h"
-#include "mbedtls/sha256.h"
+#include "psa/crypto.h"
 #include "send.h"
 
 #define COOKIE_SIZE 20
@@ -76,7 +76,7 @@ static int generate_cookie(t_session *session, char *cookie, size_t size) {
 	char text[MAX_IP_STR_LEN + SECRET_SIZE + 1];
 	unsigned char bin_hash[32];
 	char hex_hash[65];
-	size_t len;
+	size_t len, hash_len;
 
 	if (size - 1 > 64) {
 		return -1;
@@ -86,7 +86,9 @@ static int generate_cookie(t_session *session, char *cookie, size_t size) {
 	len = strlen(text);
 	memcpy(text + len, secret, SECRET_SIZE);
 	*(text + len + SECRET_SIZE) = '\0';
-	mbedtls_sha256((unsigned char*)text, len + SECRET_SIZE, bin_hash, 0);
+	if (psa_hash_compute(PSA_ALG_SHA_256, (unsigned char*)text, len + SECRET_SIZE, bin_hash, 32, &hash_len) != PSA_SUCCESS) {
+		return -1;
+	}
 	sha256_bin2hex(bin_hash, hex_hash);
 
 	memcpy(cookie, hex_hash, size - 1);
@@ -119,7 +121,7 @@ static int get_cookie(t_session *session, char *cookie, size_t size) {
 		if (strlen(value) >= size) {
 			break;
 		}
-		strcpy(cookie, value);
+		strlcpy(cookie, value, size);
 
 		free(header);
 
@@ -146,13 +148,13 @@ static int send_cookie_head(t_session *session) {
 }
 
 static int send_cookie_tail(t_session *session, int content_length) {
-	char conlen[10];
+	char conlen[20];
 
-	if (content_length > 99999) {
+	if ((content_length < 0) || (content_length > 99999)) {
 		return -1;
 	}
 
-	sprintf(conlen, "%d\r\n\r\n", content_length);
+	snprintf(conlen, sizeof(conlen), "%d\r\n\r\n", content_length);
 
 	if (send_buffer(session, hs_conlen, 16) == -1) {
 		return -1;
