@@ -22,7 +22,7 @@
 #include "libstr.h"
 #include "libfs.h"
 #include "filehashes.h"
-#include "mbedtls/sha256.h"
+#include "psa/crypto.h"
 #include "memdbg.h"
 
 #define BUFFER_SIZE 1024
@@ -30,24 +30,37 @@
 int sha256_file(const char *path, unsigned char output[32], int is224) {
 	FILE *fp;
 	size_t bytes_read;
-	mbedtls_sha256_context context;
+	psa_hash_operation_t op = PSA_HASH_OPERATION_INIT;
 	unsigned char buffer[BUFFER_SIZE];
+	psa_algorithm_t algo;
+	size_t hash_length;
 
 	if ((fp = fopen(path, "rb")) == NULL) {
 		return -1;
 	}
 
-	mbedtls_sha256_init(&context);
-	mbedtls_sha256_starts(&context, is224);
-
-	while ((bytes_read = fread(buffer, 1, BUFFER_SIZE, fp)) > 0) {
-		mbedtls_sha256_update(&context, buffer, bytes_read);
+	algo = is224 ? PSA_ALG_SHA_224 : PSA_ALG_SHA_256;
+	if (psa_hash_setup(&op, algo) != PSA_SUCCESS) {
+		fclose(fp);
+		return -1;
 	}
 
-	mbedtls_sha256_finish(&context, output);
-	mbedtls_sha256_free(&context);
+	while ((bytes_read = fread(buffer, 1, BUFFER_SIZE, fp)) > 0) {
+		if (psa_hash_update(&op, buffer, bytes_read) != PSA_SUCCESS) {
+			psa_hash_abort(&op);
+			fclose(fp);
+			return -1;
+		}
+	}
 
 	if (ferror(fp) != 0) {
+		psa_hash_abort(&op);
+		fclose(fp);
+		return -1;
+	}
+
+	if (psa_hash_finish(&op, output, 32, &hash_length) != PSA_SUCCESS) {
+		psa_hash_abort(&op);
 		fclose(fp);
 		return -1;
 	}
@@ -169,6 +182,7 @@ int print_file_hashes(char *directory) {
 	unsigned char bin_hash[SHA_HASH_SIZE];
 	char hex_hash[FILE_HASH_SIZE + 1], *file = NULL;
 	int result = -1;
+	size_t len;
 
 	if ((dp = opendir(directory)) == NULL) {
 		return -1;
@@ -179,10 +193,11 @@ int print_file_hashes(char *directory) {
 			continue;
 		}
 
-		if ((file = malloc(strlen(directory) + strlen(fileinfo->d_name) + 2)) == NULL) {
+		len = strlen(directory) + strlen(fileinfo->d_name) + 2;
+		if ((file = malloc(len)) == NULL) {
 			goto hash_fail;
 		}
-		sprintf(file, "%s/%s", directory, fileinfo->d_name);
+		snprintf(file, len, "%s/%s", directory, fileinfo->d_name);
 
 		switch (file_type(file)) {
 			case ft_dir:
