@@ -52,6 +52,12 @@
 #include "monitor.h"
 #include "memdbg.h"
 #include "challenge.h"
+#ifndef ENABLE_TLS
+#include "mbedtls/pk.h"
+#endif
+#ifdef ENABLE_SANDBOX
+#include "sandbox.h"
+#endif
 
 #define rs_NONE                  0
 #define rs_QUIT_SERVER           1
@@ -65,6 +71,9 @@ typedef struct {
 	char *config_dir;
 	bool daemon;
 	bool config_check;
+#ifdef ENABLE_SANDBOX
+	bool show_sandbox;
+#endif
 } t_settings;
 
 static volatile int received_signal = rs_NONE;
@@ -85,11 +94,14 @@ char *enabled_modules = ""
 #ifdef ENABLE_MONITOR
 	", Monitor"
 #endif
+#ifdef ENABLE_TLS
+	", mbed TLS v"MBEDTLS_VERSION_STRING
+#endif
 #ifdef ENABLE_RPROXY
 	", ReverseProxy"
 #endif
-#ifdef ENABLE_TLS
-	", mbed TLS v"MBEDTLS_VERSION_STRING
+#ifdef ENABLE_SANDBOX
+    ", Sandbox"
 #endif
 #ifdef ENABLE_TOMAHAWK
 	", Tomahawk"
@@ -308,20 +320,24 @@ void SEGV_handler() {
 	exit(EXIT_FAILURE);
 }
 
-void TERM_handler() {
+void TERM_handler(int sig) {
+	(void)sig;
 	received_signal = rs_QUIT_SERVER;
 }
 
-void HUP_handler() {
+void HUP_handler(int sig) {
+	(void)sig;
 	received_signal = rs_UNLOCK_LOGFILES;
 }
 
-void USR1_handler() {
+void USR1_handler(int sig) {
+	(void)sig;
 	received_signal = rs_UNBAN_CLIENTS;
 }
 
 #ifdef ENABLE_CACHE
-void USR2_handler() {
+void USR2_handler(int sig) {
+	(void)sig;
 	received_signal = rs_CLEAR_CACHE;
 }
 #endif
@@ -387,7 +403,7 @@ int bind_sockets(t_binding *binding) {
 			/* Handle error
 		 	 */
 			if (inet_ntop(binding->interface.family, &(binding->interface.value), ip_address, MAX_IP_STR_LEN) == NULL) {
-				strcpy(ip_address, "?.?.?.?");
+				strlcpy(ip_address, "?.?.?.?", MAX_IP_STR_LEN);
 			}
 			fprintf(stderr, "Error binding %s%c%d\n", ip_address, separator, binding->port);
 			return -1;
@@ -596,6 +612,22 @@ int run_webserver(t_settings *settings) {
 		return -1;
 	}
 
+#ifdef ENABLE_SANDBOX
+    /* Collect filesystem paths for sandbox before privilege drop */
+	if (config->enable_sandbox) {
+    	sandbox_collect_paths(config);
+	}
+
+	if (settings->show_sandbox) {
+		if (config->enable_sandbox) {
+			sandbox_print_paths();
+		} else {
+			printf("Sandbox is not enabled.\n");
+		}
+		return 0;
+	}
+#endif
+
 	if (read_mimetypes(config->mimetype_config, &(config->mimetype), settings->config_check) == -1) {
 		fprintf(stderr, "Error while reading mimetype configuration.\n");
 		return -1;
@@ -616,8 +648,11 @@ int run_webserver(t_settings *settings) {
 	if (init_tls_module(config->ca_certificates) == -1) {
 		return -1;
 	}
-
-	tls_setup.dh_size         = config->dh_size;
+#else
+	if (psa_crypto_init() != PSA_SUCCESS) {
+		fprintf(stderr, "Failed to initialize PSA Crypto.\n");
+		return -1;
+	}
 #endif
 
 	/* Load private keys and certificate for bindings
@@ -934,7 +969,7 @@ int run_webserver(t_settings *settings) {
 #endif
 		) {
 			bzero(&afa, sizeof(afa));
-			strcpy(afa.af_name, "httpready");
+			strlcpy(afa.af_name, "httpready", sizeof(afa.af_name));
 			if (setsockopt(binding->socket, SOL_SOCKET, SO_ACCEPTFILTER, &afa, sizeof(afa)) == -1) {
 				fprintf(stderr, "Error while enabling HTTP accept filter. Kernel module 'accf_http' loaded?");
 				return -1;
@@ -964,6 +999,12 @@ int run_webserver(t_settings *settings) {
 		}
 	}
 
+#ifdef ENABLE_SANDBOX
+	if (config->enable_sandbox) {
+    	sandbox_apply();
+	}
+#endif
+
 	log_system(config, "Hiawatha v"VERSION" started.");
 
 	/* Start task_runner
@@ -1125,6 +1166,9 @@ void show_help(char *hiawatha) {
 	printf("         -d: don't fork to the background.\n");
 	printf("         -h: show this information and exit.\n");
 	printf("         -k: check configuration and exit.\n");
+#ifdef ENABLE_SANDBOX
+	printf("         -s: show sandbox information and exit.\n");
+#endif
 	printf("         -m: show enabled modules and exit.\n");
 	printf("         -v: show version and copyright and exit.\n");
 }
@@ -1140,6 +1184,9 @@ int main(int argc, char *argv[]) {
 	settings.config_dir   = CONFIG_DIR;
 	settings.daemon       = true;
 	settings.config_check = false;
+#ifdef ENABLE_SANDBOX
+	settings.show_sandbox = false;
+#endif
 
 	/* Read command line parameters
 	 */
@@ -1166,6 +1213,10 @@ int main(int argc, char *argv[]) {
 			}
 			printf("Enabled modules: %s\n", enabled_modules);
 			return EXIT_SUCCESS;
+#ifdef ENABLE_SANDBOX
+		} else if (strcmp(argv[i], "-s") == 0) {
+			settings.show_sandbox = true;
+#endif
 		} else if (strcmp(argv[i], "-v") == 0) {
 			printf("%s, copyright (c) by Hugo Leisink <hugo@leisink.net>\n", version_string);
 			return EXIT_SUCCESS;
