@@ -563,7 +563,7 @@ int parse_request(t_session *session, int total_bytes) {
 /* Convert the request uri to a filename.
  */
 int uri_to_path(t_session *session) {
-	size_t length;
+	size_t length, fod_len;
 	char *strstart, *strend;
 	t_keyvalue *alias;
 	int retval;
@@ -635,13 +635,13 @@ int uri_to_path(t_session *session) {
 	/* Allocate memory
 	 */
 	if (alias == NULL) {
-		length = session->host->website_root_len;
+		fod_len = session->host->website_root_len;
 	} else {
-		length = strlen(alias->value);
+		fod_len = strlen(alias->value);
 	}
-	length += session->uri_len + MAX_START_FILE_LENGTH;
+	fod_len += session->uri_len + MAX_START_FILE_LENGTH + 4; /* + 3 for '.gz' (gzip encoding) */
 
-	if ((session->file_on_disk = (char*)malloc(length + 4)) == NULL) { /* + 3 for '.gz' (gzip encoding) */
+	if ((session->file_on_disk = (char*)malloc(fod_len)) == NULL) {
 		return 500;
 	}
 
@@ -660,7 +660,8 @@ int uri_to_path(t_session *session) {
 		strstart = session->uri + alias->key_len;
 
 	}
-	strcpy(session->file_on_disk + length, strstart);
+
+	strlcpy(session->file_on_disk + length, strstart, fod_len - length);
 
 	return 200;
 }
