@@ -30,7 +30,7 @@
 #include "monitor.h"
 #include "log.h"
 #include "mbedtls/base64.h"
-#include "mbedtls/md5.h"
+#include "psa/crypto.h"
 #include "memdbg.h"
 
 #define ha_ALLOWED   200
@@ -376,7 +376,7 @@ static int digest_http_authentication(t_session *session, char *auth_str) {
 	char *key, *value, *rest, *empty = "", *passwd, A1[33], A2[33], result[33];
 	char *username = empty, *realm = empty, *nonce = empty, *uri = empty, *response = empty;
 	//char *opaque = empty, *algoritm = empty, *cnonce = empty, *nc = empty, *qop = empty;
-	size_t len;
+	size_t len, hash_len;
 
 	key = rest = auth_str;
 	while (*key != '\0') {
@@ -414,18 +414,6 @@ static int digest_http_authentication(t_session *session, char *auth_str) {
 				uri = unquoted(value);
 			} else if (strcmp(key, "response") == 0) {
 				response = unquoted(value);
-/*
-			} else if (strcmp(key, "opaque") == 0) {
-				opaque = unquoted(value);
-			} else if (strcmp(key, "algoritm") == 0) {
-				algoritm = unquoted(value);
-			} else if (strcmp(key, "cnonce") == 0) {
-				cnonce = unquoted(value);
-			} else if (strcmp(key, "nc") == 0) {
-				nc = unquoted(value);
-			} else if (strcmp(key, "qop") == 0) {
-				qop = unquoted(value);
-*/
 			}
 		}
 		key = rest;
@@ -457,23 +445,29 @@ static int digest_http_authentication(t_session *session, char *auth_str) {
 
 	/* Calculate A2
 	 */
-	len = strlen(session->method) + strlen(uri);
-	if ((value = (char*)malloc(len + 2)) == NULL) {
+	len = strlen(session->method) + strlen(uri) + 2;
+	if ((value = (char*)malloc(len)) == NULL) {
+		return ha_ERROR;
+	}
+	snprintf(value, len, "%s:%s", session->method, uri);
+	if (psa_hash_compute(PSA_ALG_MD5, (unsigned char*)value, strlen(value), digest, 16, &hash_len) != PSA_SUCCESS) {
+		clear_free(value, strlen(value));
 		return ha_ERROR;
 	}
-	sprintf(value, "%s:%s", session->method, uri);
-	mbedtls_md5((unsigned char*)value, strlen(value), digest);
 	md5_bin2hex(digest, A2);
 	clear_free(value, strlen(value));
 
 	/* Calculate response
 	 */
-	len = strlen(A1) + strlen(nonce) + strlen(A2);
-	if ((value = (char*)malloc(len + 3)) == NULL) {
+	len = strlen(A1) + strlen(nonce) + strlen(A2) + 3;
+	if ((value = (char*)malloc(len)) == NULL) {
+		return ha_ERROR;
+	}
+	snprintf(value, len, "%s:%s:%s", A1, nonce, A2);
+	if (psa_hash_compute(PSA_ALG_MD5, (unsigned char*)value, strlen(value), digest, 16, &hash_len) != PSA_SUCCESS) {
+		clear_free(value, strlen(value));
 		return ha_ERROR;
 	}
-	sprintf(value, "%s:%s:%s", A1, nonce, A2);
-	mbedtls_md5((unsigned char*)value, strlen(value), digest);
 	md5_bin2hex(digest, result);
 	clear_free(value, strlen(value));
 
