@@ -223,7 +223,7 @@ int parse_ip_port(char *line, t_ip_addr *ip, int *port) {
  */
 int ip_to_str(t_ip_addr *ip, char *str, int max_len) {
 	if (inet_ntop(ip->family, &(ip->value), str, max_len) == NULL) {
-		strncpy(str, unknown_ip, max_len);
+		strlcpy(str, unknown_ip, max_len);
 		str[max_len - 1] = '\0';
 		return -1;
 	}
@@ -238,7 +238,7 @@ int anonymized_ip_to_str(t_ip_addr *ip, char *str, int max_len) {
 	int mask;
 
 	if (ip == NULL) {
-		strncpy(str, unknown_ip, max_len);
+		strlcpy(str, unknown_ip, max_len);
 		str[max_len - 1] = '\0';
 		return -1;
 	} else if (ip->family == AF_INET) {
@@ -246,14 +246,14 @@ int anonymized_ip_to_str(t_ip_addr *ip, char *str, int max_len) {
 	} else if (ip->family == AF_INET6) {
 		mask = 48;
 	} else {
-		strncpy(str, unknown_ip, max_len);
+		strlcpy(str, unknown_ip, max_len);
 		str[max_len - 1] = '\0';
 		return -1;
 	}
 
 	copy_ip(&anonymized_ip, ip);
 	if (apply_netmask(&anonymized_ip, mask) == -1) {
-		strncpy(str, unknown_ip, max_len);
+		strlcpy(str, unknown_ip, max_len);
 		str[max_len - 1] = '\0';
 		return -1;
 	}
