@@ -49,6 +49,30 @@ void check_clear_free(void *ptr, int size) {
 	}
 }
 
+#ifndef HAVE_STRLCPY
+size_t strlcpy(char *dst, const char *src, size_t dstsize) {
+    const char *osrc = src;
+    size_t nleft = dstsize;
+
+    if (nleft != 0) {
+        while (--nleft != 0) {
+            if ((*dst++ = *src++) == '\0') {
+                break;
+			}
+        }
+    }
+
+    if (nleft == 0) {
+        if (dstsize != 0) {
+            *dst = '\0';
+		}
+        while (*src++);
+    }
+
+    return (src - osrc - 1);
+}
+#endif
+
 /* Convert string to a boolean.
  */
 int parse_yesno(char *yesno, bool *result) {
@@ -415,6 +439,7 @@ static bool char_needs_encoding(char c) {
 int url_encode(char *str, char **encoded) {
 	char *c, *e;
 	int replaced = 0;
+	size_t len;
 
 	if (str == NULL) {
 		return -1;
@@ -431,7 +456,10 @@ int url_encode(char *str, char **encoded) {
 	if (replaced == 0) {
 		*encoded = NULL;
 		return 0;
-	} else if ((*encoded = (char*)malloc(strlen(str) + (2 * replaced) + 1)) == NULL) {
+	}
+
+	len = strlen(str) + (2 * replaced) + 1;
+	if ((*encoded = (char*)malloc(len)) == NULL) {
 		return -1;
 	}
 
@@ -439,13 +467,15 @@ int url_encode(char *str, char **encoded) {
 	e = *encoded;
 	while (*c != '\0') {
 		if (char_needs_encoding(*c)) {
-			sprintf(e, "%%%02hhx", *c);
+			snprintf(e, len, "%%%02hhx", *c);
 			e += 2;
+			len -= 2;
 		} else {
 			*e = *c;
 		}
 		c++;
 		e++;
+		len--;
 	}
 	*e = '\0';
 
@@ -506,7 +536,8 @@ bool forbidden_chars_present(char *str) {
 
 int str_replace(char *src, char *from, char *to, char **dst) {
 	char *pos, *start;
-	int replaced = 0, len_from, len_to, len_start;
+	int replaced = 0;
+	size_t len_from, len_to, len_start, len_rest;
 
 	if ((src == NULL) || (from == NULL) || (to == NULL) || (dst == NULL)) {
 		return 0;
@@ -530,7 +561,9 @@ int str_replace(char *src, char *from, char *to, char **dst) {
 		if (len_to > 0) {
 			memcpy(*dst + len_start, to, len_to);
 		}
-		strcpy(*dst + len_start + len_to, pos + len_from);
+
+		len_rest = strlen(pos + len_from) + 1;
+		memcpy(*dst + len_start + len_to, pos + len_from, len_rest);
 
 		if (replaced > 0) {
 			free(src);
@@ -575,7 +608,7 @@ int header_to_variable(char *header, char *variable, int size) {
 		return -1;
 	}
 
-	strcpy(variable, "HTTP_");
+	strlcpy(variable, "HTTP_", size);
 
 	for (i = 0; i < len; i++) {
 		if (((header[i] >= 'A') && (header[i] <= 'Z')) || ((header[i] >= '0') && (header[i] <= '9'))) {
@@ -690,7 +723,7 @@ void md5_bin2hex(unsigned char bin[16], char hex[33]) {
 	int i;
 
 	for (i = 0; i < 16; i++) {
-		sprintf(&hex[2 * i], "%02x", bin[i]);
+		snprintf(&hex[2 * i], 3, "%02x", bin[i]);
 	}
 	hex[32] = '\0';
 }
@@ -699,7 +732,7 @@ void sha256_bin2hex(unsigned char bin[32], char hex[65]) {
 	int i;
 
 	for (i = 0; i < 32; i++) {
-		sprintf(&hex[2 * i], "%02x", bin[i]);
+		snprintf(&hex[2 * i], 3, "%02x", bin[i]);
 	}
 	hex[64] = '\0';
 }
