@@ -23,6 +23,9 @@
 void check_free(void *ptr);
 void clear_free(void *ptr, int size);
 void check_clear_free(void *ptr, int size);
+#ifndef HAVE_STRLCPY
+size_t strlcpy(char *dst, const char *src, size_t dstsize);
+#endif
 
 int  parse_yesno(char *yesno, bool *result);
 int  str_to_int(char *str);
