@@ -220,7 +220,7 @@ int monitor_event(char *event, ...) {
 	int size, i;
 	va_list args;
 
-	size = i = sprintf(str, "event\t");
+	size = i = snprintf(str, sizeof(str), "event\t");
 
 	va_start(args, event);
 	size += vsnprintf(str + size, 1023 - size, event, args);
