@@ -32,7 +32,7 @@
 #include "libstr.h"
 #include "libfs.h"
 #include "libstr.h"
-#include "mbedtls/md5.h"
+#include "psa/crypto.h"
 #include "memdbg.h"
 
 #define RPROXY_ID_LEN             10 /* Must be smaller than 32 */
@@ -62,24 +62,27 @@ typedef struct type_send_buffer {
  */
 int init_rproxy_module(void) {
 	unsigned char digest[16];
-	char str[50], *format = "%s %s\r\n";
+	char str[50];
 	struct tm s;
 	time_t t;
+	size_t hash_len;
 
 	time(&t);
 	localtime_r(&t, &s);
 	strftime(str, 49, "%a %d %b %Y %T", &s);
 	str[49] = '\0';
 
-	mbedtls_md5((unsigned char*)str, strlen(str), digest);
+	if (psa_hash_compute(PSA_ALG_MD5, (unsigned char*)str, strlen(str), digest, 16, &hash_len) != PSA_SUCCESS) {
+		return -1;
+	}
 	md5_bin2hex(digest, rproxy_id);
 	rproxy_id[RPROXY_ID_LEN] = '\0';
 
-	rproxy_header_len = strlen(format) - 4 + strlen(rproxy_id_key) + RPROXY_ID_LEN;
+	rproxy_header_len = strlen(rproxy_id_key) + strlen(rproxy_id) + 3;
 	if ((rproxy_header = (char*)malloc(rproxy_header_len + 1)) == NULL) {
 		return -1;
 	}
-	sprintf(rproxy_header, format, rproxy_id_key, rproxy_id);
+	snprintf(rproxy_header, rproxy_header_len + 1, "%s %s\r\n", rproxy_id_key, rproxy_id);
 
 	return 0;
 }
@@ -535,7 +538,7 @@ int send_request_to_webserver(t_rproxy_webserver *webserver, t_rproxy_options *o
 		if (strncasecmp(http_header->data, hs_forwarded, 10) == 0) {
 			/* Add IP to Forwarded header
 			 */
-			if (sprintf(forwarded_for, ", for=\"%s\"\r\n", ip_addr) == -1) {
+			if (snprintf(forwarded_for, 20 + MAX_IP_STR_LEN, ", for=\"%s\"\r\n", ip_addr) == -1) {
 				return -1;
 			} else if (send_to_webserver(webserver, result, &send_buffer, forwarded_for, strlen(forwarded_for)) == -1) {
 				return -1;
@@ -546,7 +549,7 @@ int send_request_to_webserver(t_rproxy_webserver *webserver, t_rproxy_options *o
 			/* Add IP to X-Forwarded-For header
 			 */
 
-			if (sprintf(forwarded_for, ", %s\r\n", ip_addr) == -1) {
+			if (snprintf(forwarded_for, 20 + MAX_IP_STR_LEN, ", %s\r\n", ip_addr) == -1) {
 				return -1;
 			} else if (send_to_webserver(webserver, result, &send_buffer, forwarded_for, strlen(forwarded_for)) == -1) {
 				return -1;
@@ -561,13 +564,13 @@ int send_request_to_webserver(t_rproxy_webserver *webserver, t_rproxy_options *o
 	/* Send Forwarded and X-Forwarded-For headers
 	 */
 	if (forwarded_found == false) {
-		if (sprintf(forwarded_for, "%s for=\"%s\"\r\n", hs_forwarded, ip_addr) == -1) {
+		if (snprintf(forwarded_for, 20 + MAX_IP_STR_LEN, "%s for=\"%s\"\r\n", hs_forwarded, ip_addr) == -1) {
 			return -1;
 		} else if (send_to_webserver(webserver, result, &send_buffer, forwarded_for, strlen(forwarded_for)) == -1) {
 			return -1;
 		}
 
-		if (sprintf(forwarded_for, "%s %s\r\n", hs_x_forwarded_for, ip_addr) == -1) {
+		if (snprintf(forwarded_for, 20 + MAX_IP_STR_LEN, "%s %s\r\n", hs_x_forwarded_for, ip_addr) == -1) {
 			return -1;
 		} else if (send_to_webserver(webserver, result, &send_buffer, forwarded_for, strlen(forwarded_for)) == -1) {
 			return -1;
