@@ -385,12 +385,12 @@ int send_header(t_session *session) {
 	/* Random header
 	 */
 	if ((session->host->random_header_length > -1) && session->binding->use_tls) {
-		sprintf(random_header, "X-Random: ");
+		snprintf(random_header, MAX_RANDOM_HEADER_LENGTH + 13, "X-Random: ");
 		length = (session->host->random_header_length * ((unsigned int)rand() & MAX_RANDOM_HEADER_LENGTH_MASK)) / MAX_RANDOM_HEADER_LENGTH_MASK;
 		if (length == 0) {
 			length = 1;
 		}
-		sprintf(random_header + 10 + length, "\r\n");
+		snprintf(random_header + 10 + length, MAX_RANDOM_HEADER_LENGTH + 3 - length, "\r\n");
 
 		random = rand() & 63;
 		for (i = 0; i < length; i++) {
