@@ -30,7 +30,10 @@
 #define MAX_LENGTH_CONFIGLINE  1024
 #define MAX_CACHE_SIZE         1024
 #define MAX_UPLOAD_SIZE        2047
+
+#ifdef ENABLE_MONITOR
 #define MONITOR_HOSTNAME  "monitor"
+#endif
 
 enum t_section { syntax_error = -1, none, binding, virtual_host, directory, fcgi_server
 #ifdef ENABLE_TOOLKIT
@@ -48,15 +51,15 @@ int init_config_module(char *config_dir) {
 	size_t config_dir_len;
 	char *line;
 
-	config_dir_len = strlen(config_dir);
+	config_dir_len = strlen(config_dir) + 12;
 
 	/* Set CONFIG_DIR config variable
 	 */
-	if ((line = (char*)malloc(config_dir_len + 12)) == NULL) {
+	if ((line = (char*)malloc(config_dir_len)) == NULL) {
 		return -1;
 	}
 
-	sprintf(line, "CONFIG_DIR=%s", config_dir);
+	snprintf(line, config_dir_len, "CONFIG_DIR=%s", config_dir);
 	if (parse_keyvalue(line, &variables, "=") == -1) {
 		return -1;
 	}
@@ -65,7 +68,7 @@ int init_config_module(char *config_dir) {
 	/* Set default path to index.xslt
 	 */
 	index_xslt = line;
-	sprintf(index_xslt, "%s/index.xslt", config_dir);
+	snprintf(index_xslt, config_dir_len, "%s/index.xslt", config_dir);
 #else
 	free(line);
 #endif
@@ -399,9 +402,12 @@ t_config *default_config(void) {
 #endif
 
 #ifdef ENABLE_TLS
-	config->dh_size            = 2048;
 	config->ca_certificates    = NULL;
 #endif
+
+#ifdef ENABLE_SANDBOX
+	config->enable_sandbox     = false;
+#endif
 	return config;
 }
 
@@ -760,19 +766,19 @@ int check_configuration(t_config *config) {
 	if ((config->upload_directory = (char*)malloc(config->upload_directory_len + 1)) == NULL) {
 		return -1;
 	}
-	sprintf(config->upload_directory, "%s/upload", config->work_directory);
+	snprintf(config->upload_directory, config->upload_directory_len + 1, "%s/upload", config->work_directory);
 
 	config->gzipped_directory_len = len + 8;
 	if ((config->gzipped_directory = (char*)malloc(config->gzipped_directory_len + 1)) == NULL) {
 		return -1;
 	}
-	sprintf(config->gzipped_directory, "%s/gzipped", config->work_directory);
+	snprintf(config->gzipped_directory, config->gzipped_directory_len + 1, "%s/gzipped", config->work_directory);
 
 #ifdef ENABLE_MONITOR
 	if ((config->monitor_directory = (char*)malloc(len + 9)) == NULL) {
 		return -1;
 	}
-	sprintf(config->monitor_directory, "%s/monitor", config->work_directory);
+	snprintf(config->monitor_directory, len + 9, "%s/monitor", config->work_directory);
 
 	if ((config->monitor_enabled) && (config->first_host->next != NULL)) {
 		host = config->first_host->next;
@@ -967,6 +973,9 @@ static bool system_setting(char *key, char *value, t_config *config) {
 	t_host *monitor_host;
 	char *alist;
 #endif
+#if defined(ENABLE_MONITOR) || defined(ENABLE_RPROXY)
+	size_t len;
+#endif
 
 	if (strcmp(key, "anonymizeip") == 0) {
 		if (parse_yesno(value, &(config->anonymize_ip)) == 0) {
@@ -1120,25 +1129,18 @@ static bool system_setting(char *key, char *value, t_config *config) {
 			return true;
 		}
 
-#ifdef ENABLE_TLS
-	} else if (strcmp(key, "dhsize") == 0) {
-		if (strcmp(value, "2048") == 0) {
-			config->dh_size = 2048;
-			return true;
-		} else if (strcmp(value, "4096") == 0) {
-			config->dh_size = 4096;
-			return true;
-		} else if (strcmp(value, "8192") == 0) {
-			config->dh_size = 8192;
-			return true;
-		}
-#endif
 	} else if (strcmp(key, "exploitlogfile") == 0) {
 		if (valid_path(value)) {
 			if ((config->exploit_logfile = strdup(value)) != NULL) {
 				return true;
 			}
 		}
+#ifdef ENABLE_SANDBOX
+	} else if (strcmp(key, "enablesandbox") == 0) {
+		if (parse_yesno(value, &(config->enable_sandbox)) == 0) {
+			return true;
+		}
+#endif
 	} else if (strcmp(key, "garbagelogfile") == 0) {
 		if (valid_path(value)) {
 			if ((config->garbage_logfile = strdup(value)) != NULL) {
@@ -1219,10 +1221,11 @@ static bool system_setting(char *key, char *value, t_config *config) {
 		monitor_host->error_logfile = NULL;
 
 		rest = "allow %s, deny all";
-		if ((alist = (char*)malloc(strlen(rest) + strlen(value) + 1)) == NULL) {
+		len = strlen(rest) + strlen(value) + 1;
+		if ((alist = (char*)malloc(len)) == NULL) {
 			return false;
 		}
-		sprintf(alist, rest, value);
+		snprintf(alist, len, rest, value);
 		if ((monitor_host->access_list = parse_accesslist(alist, false, NULL)) == NULL) {
 			return false;
 		}
@@ -1403,10 +1406,11 @@ static bool system_setting(char *key, char *value, t_config *config) {
 			value = rest;
 		}
 
-		if ((config->tunnel_ssh_credential = (char*)malloc(7 + strlen(value))) == NULL) {
+		len = 7 + strlen(value);
+		if ((config->tunnel_ssh_credential = (char*)malloc(len)) == NULL) {
 			return false;
 		}
-		sprintf(config->tunnel_ssh_credential, "Basic %s", value);
+		snprintf(config->tunnel_ssh_credential, len, "Basic %s", value);
 
 		return true;
 
@@ -1773,10 +1777,10 @@ static bool host_setting(char *key, char *value, t_host *host) {
 			if ((time = time_str_to_int(rest)) <= 0) {
 				return false;
 			}
-			time = sprintf(host->hsts_time, "%d", time);
+			time = snprintf(host->hsts_time, size + 10, "%d", time);
 
 			if (value != NULL) {
-				sprintf(host->hsts_time + time, ";%s", value);
+				snprintf(host->hsts_time + time, size + 10 - time, ";%s", value);
 
 				if ((rest = strchr(value, ';')) != NULL) {
 					*rest = '\0';
