@@ -424,9 +424,12 @@ typedef struct type_config {
 #endif
 
 #ifdef ENABLE_TLS
-	int           dh_size;
 	TLS_cert      *ca_certificates;
 #endif
+
+#ifdef ENABLE_SANDBOX
+	bool          enable_sandbox;
+#endif
 } t_config;
 
 int init_config_module(char *config_dir);
