@@ -338,7 +338,7 @@ int get_homedir(t_session *session, char *username) {
 		return -1;
 	}
 
-	sprintf(session->host->website_root, "%s/%s", pwd->pw_dir, session->config->user_directory);
+	snprintf(session->host->website_root, len + 1, "%s/%s", pwd->pw_dir, session->config->user_directory);
 	if (register_tempdata(&(session->tempdata), session->host->website_root, tc_data) == -1) {
 		free(session->host->website_root);
 		session->host->website_root = old_root;
@@ -619,8 +619,8 @@ static int prevent_xss_str(t_session *session, char *input) {
 
 		if (value == '<') {
 			str++;
-			strncpy(tag, str, 21);
-			tag[21] = '\0';
+			strlcpy(tag, str, sizeof(tag));
+			tag[sizeof(tag) - 1] = '\0';
 			url_decode(tag);
 			strlower(tag);
 
