@@ -43,7 +43,7 @@
 #include "tomahawk.h"
 #include "xslt.h"
 #include "memdbg.h"
-#include "mbedtls/sha256.h"
+#include "psa/crypto.h"
 
 #define FILE_BUFFER_SIZE      32 * KILOBYTE
 #define MAX_OUTPUT_HEADER     16 * KILOBYTE
@@ -85,6 +85,7 @@ int send_file(t_session *session) {
 	int  retval, handle = -1;
 	struct stat status, gz_status;
 	struct tm fdate;
+	size_t hash_len;
 #ifdef ENABLE_CACHE
 	char *file_for_cache = NULL;
 	t_cached_object *cached_object;
@@ -117,7 +118,10 @@ int send_file(t_session *session) {
 		goto no_gzip;
 	}
 
-	mbedtls_sha256((unsigned char*)session->file_on_disk, strlen(session->file_on_disk), gz_hash, 0);
+	if (psa_hash_compute(PSA_ALG_SHA_256, (unsigned char*)session->file_on_disk, strlen(session->file_on_disk), gz_hash, 32, &hash_len) != PSA_SUCCESS) {
+		goto no_gzip;
+	}
+
 	sha256_bin2hex(gz_hash, gz_hex);
 
 	if (snprintf(gz_file, 1023, "%s/%s.gz", session->config->gzipped_directory, gz_hex) > 1020) {
