@@ -28,17 +28,15 @@
 #include "tls.h"
 #include "libstr.h"
 #include "log.h"
-#include "mbedtls/ctr_drbg.h"
-#include "mbedtls/entropy.h"
-#include "mbedtls/dhm.h"
+
 #include "mbedtls/ssl_cache.h"
 #include "mbedtls/error.h"
 #include "mbedtls/net_sockets.h"
 #include "mbedtls/x509_csr.h"
-#include "mbedtls/sha256.h"
 #include "mbedtls/base64.h"
-#include "mbedtls/error.h"
 #include "mbedtls/psa_util.h"
+#include "psa/crypto.h"
+
 #ifdef ENABLE_DEBUG
 #include "mbedtls/debug.h"
 #endif
@@ -78,33 +76,13 @@ static int ciphersuites[] = {
 	MBEDTLS_TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256,
 	MBEDTLS_TLS_ECDHE_RSA_WITH_CAMELLIA_256_GCM_SHA384,
 	MBEDTLS_TLS_ECDHE_RSA_WITH_CAMELLIA_128_GCM_SHA256,
-//	MBEDTLS_TLS_DHE_RSA_WITH_CHACHA20_POLY1305_SHA256,
-//	MBEDTLS_TLS_DHE_RSA_WITH_AES_256_GCM_SHA384,
-//	MBEDTLS_TLS_DHE_RSA_WITH_AES_128_GCM_SHA256,
-//	MBEDTLS_TLS_DHE_RSA_WITH_CAMELLIA_256_GCM_SHA384,
 	0
 };
 
-static pthread_mutex_t random_mutex;
-static pthread_mutex_t cache_mutex;
 static mbedtls_ssl_cache_context cache;
 static t_sni_list *sni_list = NULL;
-static mbedtls_ctr_drbg_context ctr_drbg;
-static mbedtls_entropy_context entropy;
 static mbedtls_ssl_config client_config;
 
-/* Required to use random number generator functions in a multithreaded application
- */
-static int tls_random(void *p_rng, unsigned char *output, size_t len) {
-	int result;
-
-	pthread_mutex_lock(&random_mutex);
-	result = mbedtls_ctr_drbg_random(p_rng, output, len);
-	pthread_mutex_unlock(&random_mutex);
-
-	return result;
-}
-
 /* TLS debug callback function
  */
 #ifdef ENABLE_DEBUG
@@ -163,21 +141,13 @@ static int sni_callback(void *param, mbedtls_ssl_context *context, const unsigne
 /* Initialize TLS library
  */
 int init_tls_module(mbedtls_x509_crt *ca_certificates) {
-	char version[16];
-
-void mbedtls_ssl_conf_dbg(mbedtls_ssl_config *conf,
-                          void (*f_dbg)(void *, int, const char *, int, const char *),
-                          void  *p_dbg);
-
-
-	if (mbedtls_version_get_number() < 0x03000000) {
-		mbedtls_version_get_string(version);
-		fprintf(stderr, "This Hiawatha installation requires at least mbed TLS v3.0.0 and you have v%s.\n", version);
+	if (psa_crypto_init() != PSA_SUCCESS) {
+		fprintf(stderr, "Failed to initialize PSA Crypto.\n");
 		return -1;
 	}
 
-	if (mbedtls_version_check_feature("MBEDTLS_THREADING_PTHREAD") != 0) {
-		fprintf(stderr, "mbed TLS was compiled without the required MBEDTLS_THREADING_PTHREAD compiler flag.\n");
+	if (mbedtls_version_get_number() < 0x04000000) {
+		fprintf(stderr, "This Hiawatha installation requires at least mbed TLS v4.0.0 and you have v%s.\n", mbedtls_version_get_string());
 		return -1;
 	}
 
@@ -185,15 +155,6 @@ void mbedtls_ssl_conf_dbg(mbedtls_ssl_config *conf,
 	mbedtls_debug_set_threshold(TLS_DEBUG_LEVEL);
 #endif
 
-	/* Entropy settings
-	 */
-	mbedtls_entropy_init(&entropy);
-	mbedtls_ctr_drbg_init(&ctr_drbg);
-	if (mbedtls_ctr_drbg_seed(&ctr_drbg, mbedtls_entropy_func, &entropy, (unsigned char*)"Hiawatha_RND", 10) != 0) {
-		return -1;
-	}
-	mbedtls_ctr_drbg_set_prediction_resistance(&ctr_drbg, MBEDTLS_CTR_DRBG_PR_OFF);
-
 	/* Cache settings
 	 */
 	mbedtls_ssl_cache_init(&cache);
@@ -206,7 +167,7 @@ void mbedtls_ssl_conf_dbg(mbedtls_ssl_config *conf,
 		return -1;
 	}
 	mbedtls_ssl_conf_renegotiation(&client_config, MBEDTLS_SSL_RENEGOTIATION_DISABLED);
-	mbedtls_ssl_conf_rng(&client_config, tls_random, &ctr_drbg);
+
 #ifdef ENABLE_DEBUG
 	mbedtls_ssl_conf_dbg(&client_config, tls_debug, &client_config);
 #endif
@@ -218,20 +179,12 @@ void mbedtls_ssl_conf_dbg(mbedtls_ssl_config *conf,
 		mbedtls_ssl_conf_ca_chain(&client_config, ca_certificates, NULL);
 	}
 
-	if (pthread_mutex_init(&random_mutex, NULL) != 0) {
-		return -1;
-	} else if (pthread_mutex_init(&cache_mutex, NULL) != 0) {
-		return -1;
-	}
-
 	return 0;
 }
 
 /* Set TLS configuration
  */
 int tls_set_config(mbedtls_ssl_config **tls_config, t_tls_setup *tls_setup) {
-	psa_status_t psa_status;
-
 	if ((*tls_config = (mbedtls_ssl_config*)malloc(sizeof(mbedtls_ssl_config))) == NULL) {
 		return -1;
 	}
@@ -245,9 +198,10 @@ int tls_set_config(mbedtls_ssl_config **tls_config, t_tls_setup *tls_setup) {
 	mbedtls_ssl_conf_max_tls_version(*tls_config, MBEDTLS_SSL_VERSION_TLS1_3);
 
 	mbedtls_ssl_conf_renegotiation(*tls_config, MBEDTLS_SSL_RENEGOTIATION_DISABLED);
-	mbedtls_ssl_conf_rng(*tls_config, tls_random, &ctr_drbg);
+
 	mbedtls_ssl_conf_sni(*tls_config, sni_callback, NULL);
 	mbedtls_ssl_conf_session_cache(*tls_config, &cache, mbedtls_ssl_cache_get, mbedtls_ssl_cache_set);
+
 #ifdef ENABLE_DEBUG
 	mbedtls_ssl_conf_dbg(*tls_config, tls_debug, &tls_config);
 #endif
@@ -263,26 +217,6 @@ int tls_set_config(mbedtls_ssl_config **tls_config, t_tls_setup *tls_setup) {
 
 	mbedtls_ssl_conf_own_cert(*tls_config, tls_setup->certificate, tls_setup->private_key);
 
-	if (tls_setup->dh_size == 2048) {
-		const unsigned char dhm_p[] = MBEDTLS_DHM_RFC7919_FFDHE2048_P_BIN;
-		const unsigned char dhm_g[] = MBEDTLS_DHM_RFC7919_FFDHE2048_G_BIN;
-		mbedtls_ssl_conf_dh_param_bin(*tls_config, dhm_p, sizeof(dhm_p), dhm_g, sizeof(dhm_g));
-	} else if (tls_setup->dh_size == 4096) {
-		const unsigned char dhm_p[] = MBEDTLS_DHM_RFC7919_FFDHE4096_P_BIN;
-		const unsigned char dhm_g[] = MBEDTLS_DHM_RFC7919_FFDHE4096_G_BIN;
-		mbedtls_ssl_conf_dh_param_bin(*tls_config, dhm_p, sizeof(dhm_p), dhm_g, sizeof(dhm_g));
-	} else if (tls_setup->dh_size == 8192) {
-		const unsigned char dhm_p[] = MBEDTLS_DHM_RFC7919_FFDHE8192_P_BIN;
-		const unsigned char dhm_g[] = MBEDTLS_DHM_RFC7919_FFDHE8192_G_BIN;
-		mbedtls_ssl_conf_dh_param_bin(*tls_config, dhm_p, sizeof(dhm_p), dhm_g, sizeof(dhm_g));
-	}
-
-	psa_status = psa_crypto_init();
-	if (psa_status != PSA_SUCCESS) {
-		mbedtls_fprintf(stderr, "Failed to initialize PSA Crypto implementation: %d\n", (int)psa_status);
-		return -1;
-	}
-
 	return 0;
 }
 
@@ -344,7 +278,7 @@ int tls_load_key_cert(char *file, mbedtls_pk_context **private_key, mbedtls_x509
 	}
 	mbedtls_pk_init(*private_key);
 
-	if ((result = mbedtls_pk_parse_keyfile(*private_key, file, NULL, mbedtls_psa_get_random, MBEDTLS_PSA_RANDOM_STATE)) != 0) {
+	if ((result = mbedtls_pk_parse_keyfile(*private_key, file, NULL)) != 0) {
 		print_tls_error(result, "Error loading private key from %s", file);
 		return -1;
 	}
@@ -441,10 +375,6 @@ int tls_accept(int *sock, mbedtls_ssl_context *context, mbedtls_ssl_config *conf
 
 	mbedtls_ssl_init(context);
 
-	if (mbedtls_ctr_drbg_reseed(&ctr_drbg, (const unsigned char*)"client thread", 13) != 0) {
-		return -1;
-	}
-
 	if (mbedtls_ssl_setup(context, config) != 0) {
 		return -1;
 	}
@@ -589,10 +519,6 @@ void tls_close(mbedtls_ssl_context *context) {
 /* Connect to remote server via TLS
  */
 int tls_connect(mbedtls_ssl_context *context, int *sock, char *hostname) {
-	if (mbedtls_ctr_drbg_reseed(&ctr_drbg, (const unsigned char*)"Reverse Proxy", 13) != 0) {
-		return -1;
-	}
-
 	mbedtls_ssl_init(context);
 
 	if (mbedtls_ssl_setup(context, &client_config) != 0) {
