@@ -27,7 +27,7 @@
 #include "client.h"
 #include "workers.h"
 #include "cache.h"
-#include "mbedtls/md5.h"
+#include "psa/crypto.h"
 #include "memdbg.h"
 
 #define MAX_IDLE_TIME   60
@@ -89,7 +89,7 @@ void show_request_to_admins(char *method, char *uri, char *http_version, t_ip_ad
 						 "  User agent: %s\n"
 				         "  Result: %d, %ld bytes sent\n\n";
 				if (snprintf(message, 1023, format, method, uri, http_version, hostname, ip_str, user_agent, response_code, (long)bytes_sent) > 1023) {
-					sprintf(message, "(error generating request information message)\n");
+					snprintf(message, sizeof(message), "(error generating request information message)\n");
 				}
 				generated = true;
 			}
@@ -399,6 +399,7 @@ static int handle_admin(t_admin *admin, t_config *config) {
 	int retval = cc_OKE;
 	char line[MAX_CMD_SIZE + 1], *pwd, encrypted[33];
 	unsigned char digest[16];
+	size_t hash_len;
 
 	if (fgets(line, MAX_CMD_SIZE, admin->fp) != NULL) {
 		line[MAX_CMD_SIZE] = '\0';
@@ -422,16 +423,21 @@ static int handle_admin(t_admin *admin, t_config *config) {
 			fprintf(admin->fp, "\033[A\033[K\033[m\n"); /* Move cursor up, clear line and reset color */
 
 			pwd = remove_spaces(line);
-			mbedtls_md5((unsigned char*)pwd, strlen(pwd), digest);
-			md5_bin2hex(digest, encrypted);
-			memset(line, 0, MAX_CMD_SIZE);
 
-			if ((admin->authenticated = (strcmp_rtap(encrypted, config->tomahawk_port->binding_id) == 0)) == false) {
+			if (psa_hash_compute(PSA_ALG_MD5, (unsigned char*)pwd, strlen(pwd), digest, 16, &hash_len) != PSA_SUCCESS) {
+				fprintf(admin->fp, "Hash error.\n\n");
 				retval = cc_DISCONNECT;
-				fprintf(admin->fp, "Password incorrect\n\n");
 			} else {
-				fprintf(admin->fp, "Welcome. Use 'help' for help. Auto-logout after %d seconds idle-time.\n\n", MAX_IDLE_TIME);
-				fflush(admin->fp);
+				md5_bin2hex(digest, encrypted);
+				memset(line, 0, MAX_CMD_SIZE);
+
+				if ((admin->authenticated = (strcmp_rtap(encrypted, config->tomahawk_port->binding_id) == 0)) == false) {
+					retval = cc_DISCONNECT;
+					fprintf(admin->fp, "Password incorrect.\n\n");
+				} else {
+					fprintf(admin->fp, "Welcome. Use 'help' for help. Auto-logout after %d seconds idle-time.\n\n", MAX_IDLE_TIME);
+					fflush(admin->fp);
+				}
 			}
 		}
 	} else {
