@@ -45,21 +45,24 @@ t_url_toolkit *select_toolkit(char *toolkit_id, t_url_toolkit *url_toolkit) {
 	return NULL;
 }
 
+/* In src, replace text at offset ofs with length len, with rep and place in dst.
+ */
 static int replace(char *src, int ofs, int len, char *rep, char **dst) {
-	size_t len_rep;
+	size_t len_rep, len_dst;
 
 	if ((src == NULL) || (rep == NULL) || (dst == NULL)) {
 		return -1;
 	}
 
 	len_rep = strlen(rep);
-	if ((*dst = (char*)malloc(strlen(src) - len + len_rep + 1)) == NULL) {
+	len_dst = strlen(src) - len + len_rep + 1;
+	if ((*dst = (char*)malloc(len_dst)) == NULL) {
 		return -1;
 	}
 
 	memcpy(*dst, src, ofs);
 	memcpy(*dst + ofs, rep, len_rep);
-	strcpy(*dst + ofs + len_rep, src + ofs + len);
+	memcpy(*dst + ofs + len_rep, src + ofs + len, len_dst - ofs - len_rep);
 
 	return 0;
 }
@@ -332,7 +335,7 @@ bool toolkit_setting(char *key, char *value, t_url_toolkit *toolkit) {
 			if ((new_rule->header = (char*)malloc(len + 2)) == NULL) {
 				return false;
 			}
-			sprintf(new_rule->header, "%s:", value);
+			snprintf(new_rule->header, len + 2, "%s:", value);
 		}
 
 		if ((*rest == '\'') || (*rest == '"')) {
