@@ -31,12 +31,12 @@
 #include "ip.h"
 #include "toolkit.h"
 #include "filehashes.h"
-#include "mbedtls/md5.h"
+#include "psa/crypto.h"
 #ifdef ENABLE_TLS
 #include "mbedtls/platform.h"
 #include "mbedtls/ssl.h"
+#include "mbedtls/pk.h"
 #include "mbedtls/x509.h"
-#include "mbedtls/psa_util.h"
 #endif
 #include "tls.h"
 
@@ -361,6 +361,7 @@ int check_main_config(char *config_dir) {
 	TLS_key private_key;
 	TLS_cert certificate;
 	char *last_file = NULL;
+	//psa_key_attributes_t pkey_attrs;
 #endif
 
 	if (quiet == false) {
@@ -582,14 +583,19 @@ int check_main_config(char *config_dir) {
 		/* Private key check
 		 */
 		mbedtls_pk_init(&private_key);
-		if (mbedtls_pk_parse_keyfile(&private_key, needle->value, NULL, mbedtls_psa_get_random, MBEDTLS_PSA_RANDOM_STATE) != 0) {
+		if (mbedtls_pk_parse_keyfile(&private_key, needle->value, NULL) != 0) {
 			printf("Error loading private key from %s.\n", needle->value);
 			errors++;
 			goto next_crt;
 		}
 
-		if ((mbedtls_pk_get_type(&private_key) == MBEDTLS_PK_RSA) && (mbedtls_pk_get_bitlen(&private_key) < RSA_MIN_SIZE)) {
-			printf("Warning: the RSA key size in %s should be at least %d bits.\n", needle->value, RSA_MIN_SIZE);
+		psa_key_attributes_t pkey_attrs = PSA_KEY_ATTRIBUTES_INIT;
+		if (mbedtls_pk_get_psa_attributes(&private_key, PSA_KEY_USAGE_SIGN_HASH, &pkey_attrs) == 0) {
+			psa_key_type_t key_type = psa_get_key_type(&pkey_attrs);
+			if (PSA_KEY_TYPE_IS_RSA(key_type) && (mbedtls_pk_get_bitlen(&private_key) < RSA_MIN_SIZE)) {
+				printf("Warning: RSA key size of %s is below %d bits.\n", needle->value, RSA_MIN_SIZE);
+			}
+			psa_reset_key_attributes(&pkey_attrs);
 		}
 
 		/* Certificate check
@@ -636,7 +642,7 @@ int add_header(t_http_header **headers, char *env_key, char *header_key) {
 		free(new);
 		return -1;
 	}
-	sprintf(new->data, "%s: %s", header_key, env_value);
+	snprintf(new->data, new->length, "%s: %s", header_key, env_value);
 	new->next = *headers;
 
 	*headers = new;
@@ -863,11 +869,11 @@ void create_basic_password(char *username, char *password) {
 		password = input;
 	}
 
-	sprintf(salt, "$%d$", HASH_ALGORITM);
+	snprintf(salt, sizeof(salt), "$%d$", HASH_ALGORITM);
 	for (i = 3; i < 19; i++) {
 		salt[i] = salt_digits[rand() % len];
 	}
-	strcpy(salt + 19, "$");
+	strlcpy(salt + 19, "$", sizeof(salt) - 19);
 	encrypted = crypt(password, salt);
 
 	fprintf(stderr, "%s:%s\n", username, encrypted);
@@ -876,18 +882,24 @@ void create_basic_password(char *username, char *password) {
 void create_digest_password(char *username, char *realm, char *password) {
 	char input[100], *data, encrypted[33];
 	unsigned char digest[16];
+	size_t data_len, hash_len;
 
 	if (password == NULL) {
 		read_password(input, 100);
 		password = input;
 	}
 
-	if ((data = (void*)malloc(strlen(username) + strlen(realm) + strlen(password) + 4)) == NULL) {
+	data_len = strlen(username) + strlen(realm) + strlen(password) + 4;
+	if ((data = (void*)malloc(data_len)) == NULL) {
 		return;
 	}
 
-	sprintf(data, "%s:%s:%s", username, realm, password);
-	mbedtls_md5((unsigned char*)data, strlen(data), digest);
+	snprintf(data, data_len, "%s:%s:%s", username, realm, password);
+	if (psa_hash_compute(PSA_ALG_MD5, (unsigned char*)data, strlen(data), digest, sizeof(digest), &hash_len) != PSA_SUCCESS) {
+		clear_free(data, strlen(data));
+		fprintf(stderr, "Error computing MD5 hash.\n");
+		return;
+	}
 	md5_bin2hex(digest, encrypted);
 
 	clear_free(data, strlen(data));
@@ -914,6 +926,11 @@ int main(int argc, char *argv[]) {
 	char *config_dir = CONFIG_DIR, *password;
 	char cwd[1024];
 
+	if (psa_crypto_init() != PSA_SUCCESS) {
+		fprintf(stderr, "Error initializing PSA crypto.\n");
+		return EXIT_FAILURE;
+	}
+
 	for (i = 1; i < argc; i++) {
 		if (strcmp(argv[i], "-b") == 0) {
 			if (++i < argc) {
@@ -981,7 +998,7 @@ int main(int argc, char *argv[]) {
 
 	errors_found += check_main_config(config_dir);
 
-	if ((quiet == false) && (errors_found == 0))  {
+	if ((quiet == false) && (errors_found == 0)) {
 		printf("No non-fatal errors found in the Hiawatha configuration.\n");
 	}
 
