@@ -137,6 +137,7 @@ static int handle_error(t_session *session, int error_code) {
 #ifdef ENABLE_XSLT
 	char *xslt_file;
 #endif
+	size_t len;
 
 	error_handler = session->host->error_handlers;
 	while (error_handler != NULL) {
@@ -157,7 +158,8 @@ static int handle_error(t_session *session, int error_code) {
 	session->vars = error_handler->parameters;
 	session->encode_gzip = false;
 
-	if ((new_fod = (char*)malloc(session->host->website_root_len + strlen(error_handler->handler) + 1)) == NULL) {
+	len = session->host->website_root_len + strlen(error_handler->handler) + 1;
+	if ((new_fod = (char*)malloc(len)) == NULL) {
 		log_error_session(session, "malloc() error while handling error");
 		return 500;
 	}
@@ -168,7 +170,8 @@ static int handle_error(t_session *session, int error_code) {
 	session->file_on_disk = new_fod;
 
 	memcpy(session->file_on_disk, session->host->website_root, session->host->website_root_len);
-	strcpy(session->file_on_disk + session->host->website_root_len, error_handler->handler);
+	len -= session->host->website_root_len;
+	strlcpy(session->file_on_disk + session->host->website_root_len, error_handler->handler, len);
 
 	if (get_target_extension(session) == -1) {
 		return 500;
@@ -894,7 +897,7 @@ no_toolkit:
 #endif
 		length = strlen(session->file_on_disk);
 		if (*(session->file_on_disk + length - 1) == '/') {
-			strcpy(session->file_on_disk + length, session->host->start_file);
+			strlcpy(session->file_on_disk + length, session->host->start_file, MAX_START_FILE_LENGTH + 1);
 
 			if (get_target_extension(session) == -1) {
 				return 500;
