@@ -49,41 +49,44 @@ extern char *unknown_http_code;
 /* Translate spacial characters
  */
 static int xml_special_chars(char *str, char **new) {
-	size_t len;
+	size_t len, d;
 	char *c;
 
 	if (str == NULL) {
 		*new = NULL;
 		return -1;
 	}
-	len = strlen(str);
 
-	if ((*new = (char*)malloc(len * 6 + 1)) == NULL) {
+	len = strlen(str) * 6 + 1;
+	if ((*new = (char*)malloc(len)) == NULL) {
 		return -1;
 	}
 	c = *new;
 
 	while (*str != '\0') {
 		if (*str == '&') {
-			strcpy(c, "&amp;");
-			c += 5;
+			strlcpy(c, "&amp;", len);
+			d = 5;
 		} else if (*str == '\"') {
-			strcpy(c, "&quot;");
-			c += 6;
+			strlcpy(c, "&quot;", len);
+			d = 6;
 		} else if (*str == '\'') {
-			strcpy(c, "&apos;");
-			c += 6;
+			strlcpy(c, "&apos;", len);
+			d = 6;
 		} else if (*str == '<') {
-			strcpy(c, "&lt;");
-			c += 4;
+			strlcpy(c, "&lt;", len);
+			d = 4;
 		} else if (*str == '>') {
-			strcpy(c, "&gt;");
-			c += 4;
+			strlcpy(c, "&gt;", len);
+			d = 4;
 		} else {
 			*c = *str;
-			c++;
+			d = 1;
 		}
 
+		c += d;
+		len -= d;
+
 		str++;
 	}
 	*c = '\0';
@@ -96,7 +99,7 @@ static int xml_special_chars(char *str, char **new) {
 /* XSLT transform parameter functions
  */
 static void add_parameter(const char **params, char *key, char *value, int *i) {
-	size_t key_len;
+	size_t key_len, str_len;
 	char *str;
 
 	if (key == NULL) {
@@ -110,7 +113,8 @@ static void add_parameter(const char **params, char *key, char *value, int *i) {
 	}
 
 	key_len = strlen(key);
-	if ((str = (char*)malloc(key_len + strlen(value) + 4)) == NULL) {
+	str_len = key_len + strlen(value) + 4;
+	if ((str = (char*)malloc(str_len)) == NULL) {
 		return;
 	}
 
@@ -124,9 +128,9 @@ static void add_parameter(const char **params, char *key, char *value, int *i) {
 	url_decode(value);
 
 	if (strchr(value, '\'') == NULL) {
-		sprintf(str, "%s%c'%s'", key, 0, value);
+		snprintf(str, str_len, "%s%c'%s'", key, 0, value);
 	} else if (strchr(value, '"') == NULL) {
-		sprintf(str, "%s%c\"%s\"", key, 0, value);
+		snprintf(str, str_len, "%s%c\"%s\"", key, 0, value);
 	} else {
 		free(value);
 		free(str);
@@ -162,7 +166,7 @@ static void add_parameter_line(const char **params, char *line, char split, char
 		}
 
 		memcpy(name, prefix, prefix_len);
-		strncpy(name + prefix_len, key, key_len + 1);
+		strlcpy(name + prefix_len, key, key_len + 1);
 
 		add_parameter(params, name, value, i);
 	}
